package jrummikub.view.impl;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.util.Collections;
import java.util.Map;

import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JPanel;

import jrummikub.model.Position;
import jrummikub.model.Stone;
import jrummikub.model.StoneSet;
import jrummikub.view.ITable;

@SuppressWarnings("serial")
public class Table extends JPanel implements ITable {
  private final static ImageIcon background = new ImageIcon(Board.class.getResource("/jrummikub/resource/felt.png"));
  
  private JLabel leftPlayerLabel, topPlayerLabel, rightPlayerLabel;
  private JPanel innerPanel;
  
  private StonePainter stonePainter = new StonePainter(1);
  
  private Map<StoneSet, Position> stoneSets = Collections.emptyMap();
  
  @Override
  public void setLeftPlayerName(String playerName) {
    leftPlayerLabel.setText(playerName);
  }

  @Override
  public void setTopPlayerName(String playerName) {
    topPlayerLabel.setText(playerName);
  }

  @Override
  public void setRightPlayerName(String playerName) {
    rightPlayerLabel.setText(playerName);
  }
  
  @Override
  public void setStoneSets(Map<StoneSet, Position> stoneSets) {
    this.stoneSets = stoneSets;
  }
  
  Table() {
    super(true); // set double buffered
    setLayout(new BorderLayout());
    
    leftPlayerLabel = new JLabel();
    leftPlayerLabel.setForeground(Color.WHITE);
    add(leftPlayerLabel, BorderLayout.WEST);
    
    topPlayerLabel = new JLabel();
    topPlayerLabel.setHorizontalAlignment(JLabel.CENTER);
    topPlayerLabel.setHorizontalTextPosition(JLabel.CENTER);
    topPlayerLabel.setForeground(Color.WHITE);
    add(topPlayerLabel, BorderLayout.NORTH);
    
    rightPlayerLabel = new JLabel();
    rightPlayerLabel.setForeground(Color.WHITE);
    add(rightPlayerLabel, BorderLayout.EAST);
    
    innerPanel = new JPanel();
    innerPanel.setOpaque(false);
    add(innerPanel, BorderLayout.CENTER);
  }
  
  @Override
  protected void paintComponent(Graphics g1) {
    Graphics2D g = (Graphics2D)g1;
    
    for(int x = 0; x < getWidth(); x += background.getIconWidth()) {
      for(int y = 0; y < getHeight(); y += background.getIconHeight()) {
        background.paintIcon(this, g, x, y);
      }
    }
    
    g.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
        RenderingHints.VALUE_ANTIALIAS_ON);

    for (Map.Entry<StoneSet, Position> stoneSet : stoneSets.entrySet()) {
      stonePainter.paintStoneSet(g, stoneSet.getKey(), stoneSet.getValue());
    }
  }
}
